#!/usr/bin/ruby
#Cody Easley

#Simple DNS script to query for Type A records from a DNS server

#Understanding and some parts taken from Pydig source code
#	http://www.huque.com/software/pydig/

#Ruby Version 1.8.6

require 'socket'

$PORT = 53
#SERVER = '149.149.134.3'		#CSC DNS server
#SERVER = '149.149.134.25'		#CSC DNS server
#SERVER = '149.149.134.24'		#CSC DNS server
SERVER = '8.8.8.8'				#Google's DNS server address

#print a binary string as hex digits for debugging
#method borrowed from Adam Chesney with permission
def printHex(s)
	for c in s:
		print "%02X" % ?c+" "
	end
	puts ""
end                

#Send the request and get the response
def sendRequest(pkt)
	soc = UDPSocket.new
	puts "send1"	
	soc.send(pkt, 0, SERVER, $PORT)	
	puts "send2"
	responsepkt, responder_addr = soc.recvfrom(1024)
	puts "send3"
	soc.close()
	puts "send4"
	return responsepkt, responder_addr
end

#Decode DNS response, process what is needed, and print IP addresses
def decodeAndProcess(pkt, sentID, request)

	#unpack the header and get it's parts out (important parts are answerID and answerCount)
	struct = pkt[0..11]
	ans = struct.unpack("nnnnnn")
	answerID = ans[0]
	answerFlags = ans[1]
	qdCount = ans[2]
	answerCount = ans[3]
	nsCount = ans[4]
	arCount = ans[5]	

	if answerID != sentID
		puts "Got response with id: %ld (expecting %ld)" % answerID, sentID
		exit(1)
	end

	if @answerCount.nil?
		puts "No addresses returned"
		exit(0)
	end

	#get the part of the message that contains the answers
	answers = pkt[len(request)..len(pkt)]

	#assumes the name in the answer is a pointer to the question
	if len(answers) == 16 * answerCount	
		#step through each answer
		for i in range(answerCount)
			
			#get the next set of numbers for the IP address
			numbers = answers[12..16]

			#get the next answer for the next loop iteration
			answers = answers[16..len(answers)]
			
			#unpack the current answer to get each IP part
			struct = [numbers]
			n = struct.unpack('BBBB')
			n1 = n[0]
			n2 = n[1]
			n3 = n[2]
			n4 = n[3]
			
			#print the formatted IP address
			print str(n1) + '.' + str(n2) + '.' + str(n3) + '.' + str(n4)
		end
	end
end

#make the DNS request, given the query and an ID
def makeRequest(query, id)

	struct = [1]

	qType = struct.pack("n")					# type = A
	qClass = struct.pack("n")					# class = IN
	qr = 0                                      # query/response
	opcode = 0                                  # standard query
	aa = 0                                      # authoritative answer
	tc = 0                                      # truncated response
	rd = 1                                      # recursion desired
	ra = 0                                      # recursion available
	z = 0                                       # reserved
	ad = 0                                      # authenticated data
	cd = 1                                      # checking disabled
	rcode = 0                                   # response code
	qdcount = struct.pack("n")	              	# 1 question
	
	struct = [0]
	
	ancount = struct.pack("n")              # 0 answer
	nscount = struct.pack("n")              # 0 authority
	arcount = struct.pack("n")

	queryAddr = ""
	for label in query.split('.')
		struct = [label.length]
		queryAddr += struct.pack("C") + label
	end

	flags = (qr << 15) + (opcode << 11) + (aa << 10) + (tc << 9) + \
		(rd << 8) + (ra << 7) + (z << 6) + (ad << 5) + (cd << 4) + rcode
	struct = [flags]
	flags = struct.pack("n")

	question = queryAddr + qType + qClass
	msg = id + flags + qdcount + ancount + nscount + arcount + question

	return msg
end

#make sure there is only command line argument
if ARGV.length != 1
	print 'This script requires one command line arguement: the web address to lookup.'
	exit(1)
else
	query = ARGV[0]
end
	
puts "Using server: \t" + SERVER
puts "Querying for: \t" + query
	
if not query[query.length - 1] == "."
	query += "."
end

#make up a random query/response id and construct the message to be passed to the DNS server
id = 1 + rand(65536)
struct = [id]
packedID = struct.pack("n")
printHex(packedID)
request = makeRequest(query,packedID)

puts "test1"

#send the request and get the response
responsePkt, addr = sendRequest(request) 

puts "test2"

if not responsePkt
	puts "No response from server"
	exit(1)
end

#formatting print statement
puts ""

puts "test3"

decodeAndProcess(responsePkt, id, request)

