#!/usr/bin/env ruby
#Justin Groce
#Perform a simple health check of the system
#Compiled from scripts written by Timothy Myers

require 'socket'
include Socket::Constants
require 'rubygems'
require 'net/ldap'
require "open3"
include Open3

#what system
win = false #linux
begin
	f = IO.popen("uname")
	a = f.readlines
	if a.length == 0
		win = true
	end
rescue Exception=>e
	win = true;
end

#--------------------------------------------------
#--------------------------------------------------
# DNS
#--------------------------------------------------
#--------------------------------------------------

# This script gets the address(es) for the domain name on the command line

hostname = '149.149.134.25'
port = 53

name = "Google.com"

#Open the socket for the dns server
s = UDPSocket.open()
s.connect(hostname, port)

idn = [1]
# id of 1
send = idn.pack('n')
# bit set for recursion requested
send += "\1\0"
# 1 question
header = [1,0,0,0]
send += header.pack('n4')
# formatting name correctly
parts = name.split('.')
for i in 0...parts.length
   arr = [parts[i].length.to_s]
   send += arr.pack("h") + parts[i]
end
arr = ["0"]
send += arr.pack('b')
# query class and type
arr = [1,1]
send += arr.pack('n2')

#Send and get back response
s.send(send,0)
data  = s.recvfrom(1024)
if data
	data = data[0]

	head = data[0,12].unpack('n6')
	ans = head[3]
	# remove the header and question
	data = data[send.length,data.length]
	if ans == 0
		puts "DNS is NOT working correctly: There are no addresses"
	elsif data.length == 16 * ans
		puts "DNS is working correctly"
	else
		puts "DNS is NOT working correctly: Answer not recognizable"
	end
else
	puts "DNS is NOT working correctly"

end
s.close
	
#--------------------------------------------------
#--------------------------------------------------
# LDAP
#--------------------------------------------------
#--------------------------------------------------

if win #windows
	f = IO.popen("dsquery user domainroot -name *")
	a = f.readlines
	if a.length > 0
		puts "AD is working correctly"
	else
		puts "AD is  NOT working correctly"
	end
else #linux
	name = ""

	treebase = "cn=Users,dc=CSC3550-3,dc=local"
	l = Net::LDAP.new :host => "149.149.135.62", :auth => {:method => :simple, :username => "cn=Timothy E. Myers,#{treebase}", :password => "P0wr%B4sh$"}

	if l.bind
		x = 0
		l.search(:base => treebase, :scope => Net::LDAP::SearchScope_SingleLevel) do |entry|
			x = x + 1
		end
		if x > 0
			puts "LDAP is working correctly"
		else
			puts "LDAP is NOT working correctly"
		end
	else
		puts "LDAP is NOT working correctly: bind failed"
	end	
end
	
#--------------------------------------------------
#--------------------------------------------------
# NTP
#--------------------------------------------------
#--------------------------------------------------

hostname = 'time.tntech.edu'
port = 123

#Open the socket for the ntp server
s = UDPSocket.open()
s.connect(hostname, port)

#We need the epoch again
epoch = 2208988800

#Again 3 for version and 4 for mode
data = "\x1b" +  "\0" * 47

#Send and get back response
s.send(data,0)
data  = s.recvfrom(1024)
if data
	#Get time back
	time_packet = data[0].unpack('N12')
	time_now = time_packet[10]
	#Correct from epoch
	time_now -= epoch

	recTime = Time.at(time_now)
	sysTime = Time.new
	ntp = "NTP is working correctly"
	if !(recTime.year == sysTime.year)
		ntp =  "NTP is NOT working correctly: year doesn't match"
	elsif !(recTime.month == sysTime.month)
		ntp =  "NTP is NOT working correctly: month doesn't match"
	elsif !(recTime.day == sysTime.day)
		ntp =  "NTP is NOT working correctly: day doesn't match"
	elsif !(recTime.hour == sysTime.hour)
		ntp =  "NTP is NOT working correctly: hour doesn't match"
	end
	puts ntp
else
	puts "NTP is NOT working correctly: Incorrect return"
end
s.close

#--------------------------------------------------
#--------------------------------------------------
# File System Status
#--------------------------------------------------
#--------------------------------------------------

if win #windows
	puts "To test File System Status, run script on linux box"
else #linux
	fss = false
	begin
		File.open("/etc/mtab") do |io|
			io.each { |line|
				line.chomp!;
				if line.include? "149.149.134.35"
					fss = true
				end
			}
		end
	rescue Exception=>e
	end
	if fss
		puts "File System is working correctly"
	else 
		puts "File System is NOT working correctly"
	end
end


#--------------------------------------------------
#--------------------------------------------------
# File-Sharing Status
#--------------------------------------------------
#--------------------------------------------------

if win #windows
	puts "To test File-Sharing Status, run script on linux box"
else #linux
	begin
		#f = IO.popen("touch testFileSharing.txt")
		#a = f.readlines
		#if a.length > 0
		#	puts "File-Sharing is NOT working correctly"
		#else
		#	system "rm testFileSharing.txt"
		#	puts "File-Sharing is working correctly"
		#end
		
		stdin, stdout, stderr = popen3('touch testFileSharing.txt')
		e = stderr.readlines
		if e.length > 0
			puts "File-Sharing is NOT working correctly"
		else
			system "rm testFileSharing.txt"
			puts "File-Sharing is working correctly"
		end
	rescue Exception=>e
		puts "File-Sharing is NOT working correctly"
	end

end
print "\n"
