#!/usr/bin/env python
# Name: Timothy Myers
# This script takes people of the form:
# Full Name,ID number,e-mail address,{home phone number; cell phone number; work number},Department
# and adds them to a unix system using the useradd command.
#
# This is a Python 2.6 script


import sys
from subprocess import Popen

# read all of the people
people = sys.stdin.read()
# split people into lines
people = people.splitlines()
# the file to write invalid lines to
errorfile = 'proc_errors.txt'
err = open(errorfile,'w')
# boolean to inform the user when an invalid line has been seen
error = False



for person in people:

   fields = person.split(',')
   # if there are not 5 csv fields the person is invalid
   if len(fields) != 5:
      err.write(person + "\n")
      error = True
      continue

   #field 1 - names
   if 1 < len(fields[0].split()) < 7:
      # again checking for suffixes
      suffix = ['jr','junior','jr.','sr','sr.','senior','i','ii','iii','iv','v','1','2','3','4','5']
      invalid = False
      # if the number of 'names' is 2, then the second must not be a suffix
      if len(fields[0].split()) == 2:
         name = fields[0].split()[1]
         comp = name.lower()
         if suffix.count(comp) == 0:
            invalid = True
            break
      # also if the number of 'names' is 6, then the sixth must be a suffix
      elif len(fields[0].split()) == 6:
         name = fields[0].split()[5]
         comp = name.lower()
         if suffix.count(comp) > 0:
            invalid = True 
            break
      if invalid:
         err.write(person + "\n")
         error = True
         continue      
      names = fields[0]
   # only one 'name' or more than 6
   else:
      err.write(person + "\n")
      error = True
      continue
   #field 2 - id#
   # if id is not a digit, invalid
   if not fields[1].isdigit():
      err.write(person + "\n")
      error = True
      continue
   idnum = fields[1]
   #field 3 - e-mail address
   email = fields[2].split('@')
   # if there is not '@' symbol, invalid
   if len(email) != 2:
      err.write(person + "\n")
      error = True
      continue
   # if the part after the '@' does not have at least one '.', invalid
   if len(email[1].split('.')) < 2:
      err.write(person + "\n")
      error = True
      continue
   email = email[0].split('.')
   login = ''
   # we get the login by just removing the dots in the email
   for i in range(len(email)):
      login += email[i].lower()
   email = fields[2]
   #field 4 - phone numbers
   #remove {} delimiters
   pnums = fields[3].strip('{}')
   # break into 3 numbers
   pnums = pnums.split(';')
   # if there aren't 3 numbers, invalid
   if len(pnums) != 3:
      err.write(person + "\n")
      error = True
      continue
   invalid = False
   # for each number
   for i in range(3):
      num = pnums[i]
      numbernumbers = 0
      # I strip characters off to determine correctness
      while len(num) > 0:
         # pull digits off of the ends
         num2 = num.strip('0123456789')
         # add difference in lengths to the number of numbers found
         numbernumbers += len(num) - len(num2)
         # if there is no difference, invalid, because there is a letter
         if len(num) == len(num2):
            invalid = True
            break
         # remove any special character markings which may seperate numbers
         num = num2.strip('.()-#&/:_ ')
      #only use if the number of digits is 10 or 11
      if not (9 < numbernumbers < 12):
         invalid = True
      if invalid:
         err.write(person +"\n")
         error = True
         break
      # stuff into variables passed upon pass
      if i == 0:
         homenum = pnums[i] 
      elif i == 1:
         cellnum = pnums[i]
      else:
         worknum = pnums[i]
   if invalid:
      continue
   #field 5 - department name
   dptname = fields[4]
   
   add = 'useradd -m '+login
   pswd = 'echo '+login+':'+idnum+' | chpasswd'
   fn = "chfn -f '"+names+"' -w '"+worknum+"' -h '"+homenum+"' "+login
   plan = 'echo "ID number: '+idnum+'\n Email Address: '+email+'\n Cell Phone: '+cellnum+'\n Department: '+dptname+'\n" > /home/'+login+'/.plan'
   try:
      # add user to the system
      proc = Popen(add, shell=True)
      proc.wait()
   except:
      print "Command "+add+" error'd out."

   try:
      # add idnumber as password
      proc = Popen(pswd, shell=True)
      proc.wait()
   except:
      print "Command "+pswd+" error'd out."

   try:
      # input some of the data
      proc = Popen(fn, shell=True)
      proc.wait()
   except:
      print "Command "+fn+" error'd out."
   try:
      # put the rest of the data into file for finger to read
      proc = Popen(plan, shell=True)
      proc.wait()
   except:
      print "Command "+plan+" error'd out."

if error:
   print "Some people were not added, please check proc_errors.txt for invalid lines"

err.close()
